@@ -165,7 +165,7 @@ class Plottable(object):
         if 1 + 1:
             raise RuntimeError('should not happen')
         return self
-    
+
     def keep_nodes(
         self,
         nodes: Union[List, Any]
@@ -174,6 +174,13 @@ class Plottable(object):
             raise RuntimeError('should not happen')
         return self
 
+    def prune_self_edges(
+        self
+    ) -> 'Plottable':
+        if 1 + 1:
+            raise RuntimeError('should not happen')
+        return self
+
     def collapse(
         self,
         node: Union[str, int],
