@@ -1,7 +1,7 @@
 from typing import Any, Callable, cast, Iterable, List, Optional, Set, Union, TYPE_CHECKING
 import math, pandas as pd
 from .Plottable import Plottable
-from .layout import SugiyamaLayout
+from .layout import SugiyamaLayout, group_in_a_box_layout as group_in_a_box_layout_base
 from .layout.graph import Graph
 from .util import deprecated, setup_logger
 logger = setup_logger(__name__)
@@ -17,6 +17,10 @@ class LayoutsMixin(MIXIN_BASE):
     def __init__(self, *args, **kwargs):
         pass
 
+    def group_in_a_box_layout(self, *args, **kwargs):
+        return group_in_a_box_layout_base(self, *args, **kwargs)
+    group_in_a_box_layout.__doc__ = group_in_a_box_layout_base.__doc__
+
     def tree_layout(self,
                     level_col: Optional[str] = None,
                     level_sort_values_by: Optional[Union[str, List[str]]] = None,
