@@ -5,11 +5,11 @@ from .layouts import LayoutsMixin
 from .feature_utils import FeatureMixin  # type: ignore
 from .dgl_utils import DGLGraphMixin  # type: ignore
 from .umap_utils import UMAPMixin  # type: ignore
-
+from .text_utils import SearchToGraphMixin  # type: ignore
 
 mixins = ([
-    CosmosMixin, NeptuneMixin, GremlinMixin, LayoutsMixin,
-    DGLGraphMixin, UMAPMixin, FeatureMixin,
+    CosmosMixin, NeptuneMixin, GremlinMixin, LayoutsMixin, 
+    SearchToGraphMixin, DGLGraphMixin, UMAPMixin, FeatureMixin,
     ComputeMixin, PlotterBase, object
 ])
 
@@ -23,6 +23,7 @@ class Plotter(  # type: ignore
         FeatureMixin.__init__(self, *args, **kwargs)
         DGLGraphMixin.__init__(self, *args, **kwargs)
         UMAPMixin.__init__(self, *args, **kwargs)
+        SearchToGraphMixin.__init__(self, *args, **kwargs)
         LayoutsMixin.__init__(self, *args, **kwargs)
         GremlinMixin.__init__(self, *args, **kwargs)
         CosmosMixin.__init__(self, *args, **kwargs)
